/* ***** BEGIN LICENCE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public Licence Version
 * 1.1 (the "Licence"); you may not use this file except in compliance with
 * the Licence. You may obtain a copy of the Licence at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the Licence
 * for the specific language governing rights and limitations under the
 * Licence.
 *
 * The Original Code is Bible Refalizer Add-on for Firefox.
 *
 * The Initial Developer of the Original Code is James Anderson.
 * Portions created by the Initial Developer are Copyright (C) 2007-2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public Licence Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public Licence Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENCE BLOCK ***** */

 /*
 * This file contains the code to handle browser-based events.
 */

// Handles browser loading
//
function Refalizer_onLoad(event) {
}

// Handles browser unloading
//
function Refalizer_onUnload(event) {
}

// Handles page loading
//
function Refalizer_onPageLoad(event) {
	var doc = event.target;

	//doc.addEventListener("pagehide", Refalizer_onPageUnload, false);

	// Get preferences branch
	var prefs = Refalizer_getPrefsBranch();

	// Get preferences
	var enabled = prefs.getBoolPref("enabled");
	var disableOnLoad = prefs.getBoolPref("disableOnLoad");

	if (disableOnLoad) {
		prefs.setBoolPref("enabled", enabled = false);
	}

	// Update status
	Refalizer_updateStatus(enabled);

	if (enabled) {
		// Refalize page (in a separate thread)
		window.setTimeout(Refalizer_refalize, 0, doc);
	}
}

// Handles page unloading
//
function Refalizer_onPageUnload(event) {
}

// Refalizes page
//
function Refalizer_refalize(doc) {
	// Get preferences branch
	var prefs = Refalizer_getPrefsBranch();

	// Get preferences
	var sID = prefs.getIntPref("site");
	var vID = prefs.getIntPref("version");
	var openInNewWindow = prefs.getBoolPref("openInNewWindow");
	var acceptPeriodSep = prefs.getBoolPref("acceptPeriodSep");

	// Get site/version info
	var info = Refalizer_getSiteVersionInfo(sID, vID);
	var excludeURL = info.site.excludeURL;
	var refSearchURL = info.site.refSearchURL;
	var refFixType = info.site.refFixType;
	var siteName = info.site.name;
	var versionName = info.version.name;
	var versionAbbr = info.version.abbr;
	var versionCode = info.version.code;
	var target = openInNewWindow ? "_blank" : "_self";

	// Check to see whether this page should be excluded
	if (doc.location.href.indexOf(excludeURL) == 0) {
		return;
	}

	// Get all text nodes in document
	var textNodes = doc.evaluate(
			"//text()",
			doc,
			null,
			XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE,
			null);

	// Get regular expression for matching references
	var refRegExp = acceptPeriodSep ? Refalizer_REGEXP_WP : Refalizer_REGEXP;

	for (var i = 0; i < textNodes.snapshotLength; i++) {
		var textNode = textNodes.snapshotItem(i);
		var textNode2 = null;
		var parNode = textNode.parentNode;
		var lastBook = null;
		var matches = null;
		if (parNode.nodeName.toLowerCase() != "a") {
			// Search for references in text
			while (matches = refRegExp.exec(textNode.data)) {
				// Get text of reference
				var ref = matches[0];
				// Get text before and after reference
				var textBeforeRef = matches.input.substring(0, matches.index);
				var textAfterRef = matches.input.substring(matches.index + ref.length);
				// Compress whitespace in text of reference
				ref = ref.replace(/\s+/g, " ");

				if (matches[1]) {
					// Get text of book (for subsequent partial references)
					lastBook = matches[1];
					// Remove extra whitespace from text of book
					lastBook = lastBook.replace(/\s+/g, " ");
					// Update text node to contain text before reference
					textNode.data = textBeforeRef;
					// Add hyperlink element containing reference
					var elemNode = doc.createElement("a");
					var href = refSearchURL
							.replace("%R", Refalizer_getFixedReference(ref, refFixType))
							.replace("%V", versionCode);
					var title = ref
							+ " (" + siteName + ": " + versionAbbr + ")";
					elemNode.setAttribute("href", href);
					elemNode.setAttribute("title", title);
					elemNode.setAttribute("target", target);
					elemNode.innerHTML = ref;
					parNode.insertBefore(elemNode, textNode.nextSibling);
					// Add second text node containing text after reference
					textNode2 = doc.createTextNode(textAfterRef);
					parNode.insertBefore(textNode2, elemNode.nextSibling);
				} else if (lastBook != null) {
					// Update text node to contain text before reference
					textNode.data = textBeforeRef;
					// Add hyperlink element containing reference
					var elemNode = doc.createElement("a");
					var href = refSearchURL
							.replace("%R", Refalizer_getFixedReference(lastBook + " " + ref, refFixType))
							.replace("%V", versionCode);
					var title = lastBook + " " + ref
							+ " (" + siteName + ": " + versionAbbr + ")";
					elemNode.setAttribute("href", href);
					elemNode.setAttribute("title", title);
					elemNode.setAttribute("target", target);
					elemNode.innerHTML = ref;
					parNode.insertBefore(elemNode, textNode.nextSibling);
					// Add second text node containing text after reference
					textNode2 = doc.createTextNode(textAfterRef);
					parNode.insertBefore(textNode2, elemNode.nextSibling);
				} else {
					// Update text node to contain text up to end of reference
					textNode.data = textBeforeRef + ref;
					// Add second text node containing text after reference
					textNode2 = doc.createTextNode(textAfterRef);
					parNode.insertBefore(textNode2, textNode.nextSibling);
				}

				// Continue to search for references in text after reference
				textNode = textNode2;
			}
		}
	}
}

// Handles icon click
//
function Refalizer_handleIconClick(event) {
	// Get preferences branch
	var prefs = Refalizer_getPrefsBranch();

	switch (event.button) {

	case 0: // left-click

		// Update preference
		var enabled = prefs.getBoolPref("enabled");
		prefs.setBoolPref("enabled", enabled = !enabled);

		// Update status
		Refalizer_updateStatus(enabled);

		if (enabled) {
			// Refalize page (in a separate thread)
			window.setTimeout(Refalizer_refalize, 0, content.document);
		} else if (prefs.getBoolPref("reloadOnDisable")) {
			// Reload page
			gBrowser.reload();
		}

		break;

	case 2: // right-click

		// Open options dialog
		Refalizer_openOptionsDialog();

		break;

	default: // anything else

	}
}

// Opens options dialog
//
function Refalizer_openOptionsDialog() {
	window.openDialog("chrome://refalizer/content/options.xul",
			"refalizer-options", "chrome,centerscreen");
}

// Add listeners for browser loading/unloading and page loading
/*
window.addEventListener("load", Refalizer_onLoad, false);
window.addEventListener("unload", Refalizer_onUnload, false);
*/
window.addEventListener("DOMContentLoaded", Refalizer_onPageLoad, false);
