/* ***** BEGIN LICENCE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public Licence Version
 * 1.1 (the "Licence"); you may not use this file except in compliance with
 * the Licence. You may obtain a copy of the Licence at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the Licence
 * for the specific language governing rights and limitations under the
 * Licence.
 *
 * The Original Code is Bible Refalizer Add-on for Firefox.
 *
 * The Initial Developer of the Original Code is James Anderson.
 * Portions created by the Initial Developer are Copyright (C) 2007-2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public Licence Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public Licence Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENCE BLOCK ***** */

 /*
 * This file contains the code to handle the options dialog.
 */

// Handle options dialog initialization
//
function Refalizer_optionsInit() {
	// Get preferences branch
	var prefs = Refalizer_getPrefsBranch();

	// Get preferences
	var sID = prefs.getIntPref("site");
	var vID = prefs.getIntPref("version");
	var acceptPeriodSep = prefs.getBoolPref("acceptPeriodSep");
	var reloadOnDisable = prefs.getBoolPref("reloadOnDisable");
	var reloadOnBibleChange = prefs.getBoolPref("reloadOnBibleChange");
	var disableOnLoad = prefs.getBoolPref("disableOnLoad");
	var openInNewWindow = prefs.getBoolPref("openInNewWindow");

	// Set component values
	document.getElementById("refalizer-accept-period-sep").checked = acceptPeriodSep;
	document.getElementById("refalizer-reload-on-disable").checked = reloadOnDisable;
	document.getElementById("refalizer-reload-on-bible").checked = reloadOnBibleChange;
	document.getElementById("refalizer-disable-on-load").checked = disableOnLoad;
	document.getElementById("refalizer-new-window").checked = openInNewWindow;

	// Update selection menus
	Refalizer_updateSiteMenu(sID);
	Refalizer_updateVersionMenu(vID);
}

// Handle options dialog OK button
//
function Refalizer_optionsOK() {
	// Get preferences branch
	var prefs = Refalizer_getPrefsBranch();

	// Get component values
	var sID = document.getElementById("refalizer-site").value;
	var vID = document.getElementById("refalizer-version").value;
	var acceptPeriodSep = document.getElementById("refalizer-accept-period-sep").checked;
	var reloadOnDisable = document.getElementById("refalizer-reload-on-disable").checked;
	var reloadOnBibleChange = document.getElementById("refalizer-reload-on-bible").checked;
	var disableOnLoad = document.getElementById("refalizer-disable-on-load").checked;
	var openInNewWindow = document.getElementById("refalizer-new-window").checked;

	var bibleChanged = (sID != prefs.getIntPref("site"))
			|| (vID != prefs.getIntPref("version"));

	// Set preferences
	prefs.setIntPref("site", sID);
	prefs.setIntPref("version", vID);
	prefs.setBoolPref("acceptPeriodSep", acceptPeriodSep);
	prefs.setBoolPref("reloadOnDisable", reloadOnDisable);
	prefs.setBoolPref("reloadOnBibleChange", reloadOnBibleChange);
	prefs.setBoolPref("disableOnLoad", disableOnLoad);
	prefs.setBoolPref("openInNewWindow", openInNewWindow);

	if (bibleChanged && reloadOnBibleChange && prefs.getBoolPref("enabled")) {
		// Reload page
		var win = window.opener;
		while (win.gBrowser == null) {
			win = win.opener;
		}
		win.gBrowser.reload();
	}

	return true;
}

// Handle options dialog Cancel button
//
function Refalizer_optionsCancel() {
	return true;
}

// Updates the 'Site' selection menu
//
function Refalizer_updateSiteMenu(sID) {
	// Get popup menu element
	var menupopup = document.getElementById("refalizer-site-menupopup");

	// Remove any existing menu items
	while (menupopup.firstChild) {
		menupopup.removeChild(menupopup.firstChild);
	}

	// Add menu items
	for (var sIndex in Refalizer_SITES) {
		var label = Refalizer_SITES[sIndex].name;

		// Create new menu item
		var menuitem = document.createElementNS(
				"http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
				"menuitem");
		menuitem.setAttribute("label", label);
		menuitem.setAttribute("value", Refalizer_SITES[sIndex].id);

		// Add menu item
		menupopup.appendChild(menuitem);
	}

	// Select correct item
	menupopup.parentNode.value = sID;
}

// Updates the 'Version' selection menu
//
function Refalizer_updateVersionMenu(vID) {
	// Get site index
	var sIndex = document.getElementById("refalizer-site").selectedIndex;

	// Get popup menu element
	var menupopup = document.getElementById("refalizer-version-menupopup");

	// Remove any existing menu items
	while (menupopup.firstChild) {
		menupopup.removeChild(menupopup.firstChild);
	}

	// Add menu items
	for (var vIndex in Refalizer_SITES[sIndex].versions) {
		var label = Refalizer_SITES[sIndex].versions[vIndex].name
				+ " [" + Refalizer_SITES[sIndex].versions[vIndex].abbr + "]";

		// Create new menu item
		var menuitem = document.createElementNS(
				"http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
				"menuitem");
		menuitem.setAttribute("label", label);
		menuitem.setAttribute("value", Refalizer_SITES[sIndex].versions[vIndex].id);

		// Add menu item
		menupopup.appendChild(menuitem);
	}

	// Select correct item
	menupopup.parentNode.value = vID;
}
