/* ***** BEGIN LICENCE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public Licence Version
 * 1.1 (the "Licence"); you may not use this file except in compliance with
 * the Licence. You may obtain a copy of the Licence at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the Licence
 * for the specific language governing rights and limitations under the
 * Licence.
 *
 * The Original Code is Bible Refalizer Add-on for Firefox.
 *
 * The Initial Developer of the Original Code is James Anderson.
 * Portions created by the Initial Developer are Copyright (C) 2007-2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public Licence Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public Licence Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENCE BLOCK ***** */

 /*
 * This file contains the code for various utility functions.
 */

// Sites/versions are arranged in the order they appear in the options menus
const Refalizer_SITES = [
	{
		id: 0,
		name: "Bible Gateway",
		excludeURL: "http://www.biblegateway.com",
		refSearchURL: "http://www.biblegateway.com/passage/?search=%R&version=%V",
		refFixType: 0,
		versions: [
			{ id: 10, name: "21st Century King James Version", abbr: "KJV21", code: "KJ21" },
			{ id: 3, name: "Amplified Bible", abbr: "AMP", code: "AMP" },
			{ id: 11, name: "American Standard Version", abbr: "ASV", code: "ASV" },
			{ id: 7, name: "Contemporary English Version", abbr: "CEV", code: "CEV" },
			{ id: 13, name: "Darby Translation", abbr: "DARBY", code: "DARBY" },
			{ id: 6, name: "English Standard Version", abbr: "ESV", code: "ESV" },
			{ id: 15, name: "Holman Christian Standard Bible", abbr: "HCSB", code: "HCSB" },
			{ id: 5, name: "King James Version", abbr: "KJV", code: "KJV" },
			{ id: 1, name: "New American Standard Bible", abbr: "NASB", code: "NASB" },
			{ id: 9, name: "New Century Version", abbr: "NCV", code: "NCV" },
			{ id: 16, name: "New International Reader's Version", abbr: "NIRV", code: "NIRV" },
			{ id: 0, name: "New International Version", abbr: "NIV", code: "NIV" },
			{ id: 17, name: "New International Version - UK", abbr: "NIVUK", code: "NIVUK" },
			{ id: 8, name: "New King James Version", abbr: "NKJV", code: "NKJV" },
			{ id: 14, name: "New Life Version", abbr: "NLV", code: "NLV" },
			{ id: 4, name: "New Living Translation", abbr: "NLT", code: "NLT" },
			{ id: 2, name: "The Message", abbr: "MSG", code: "MSG" },
			{ id: 18, name: "Today's New International Version", abbr: "TNIV", code: "TNIV" },
			{ id: 12, name: "Young's Literal Translation", abbr: "YLT", code: "YLT" }
		]
	},
	{
		id: 6,
		name: "Bible.Logos.com",
		excludeURL: "http://bible.logos.com",
		refSearchURL: "http://bible.logos.com/passage/%V/%R",
		refFixType: 1,
		versions: [
			{ id: 7, name: "1881 Westcott-Hort Greek New Testament", abbr: "WH-GNT", code: "WESTCOTT" },
			{ id: 8, name: "1890 Darby Bible", abbr: "DARBY", code: "DARBY" },
			{ id: 9, name: "American Standard Version", abbr: "ASV", code: "ASV" },
			{ id: 13, name: "Arabic Bible (Smith & Van Dyke)", abbr: "ARABIC", code: "AR-VANDYKE" },
			{ id: 11, name: "Cambridge Paragraph Bible of the Authorized Version", abbr: "CPB-AV", code: "AV 1873" },
			{ id: 14, name: "Elzevir Textus Receptus (1624)", abbr: "ELZEVIR", code: "ELZEVIR" },
			{ id: 1, name: "English Standard Version", abbr: "ESV", code: "ESV" },
			{ id: 15, name: "Giovanni Diodati Bibbia", abbr: "DIODATI", code: "IT-DIODATI1649" },
			{ id: 26, name: "GOD'S WORD Translation", abbr: "GW", code: "GW" },
			{ id: 23, name: "Greek New Testament: Byzantine Textform 2005", abbr: "BYZ", code: "BYZ" },
			{ id: 24, name: "Holman Christian Standard Bible", abbr: "HCSB", code: "HCSB" },
			{ id: 4, name: "King James Version", abbr: "KJV", code: "KJV" },
			{ id: 30, name: "NET Bible", abbr: "NET", code: "NET" },
			{ id: 25, name: "New American Standard Bible: 1995 Update", abbr: "NASB", code: "NASB" },
			{ id: 0, name: "New International Version", abbr: "NIV", code: "NIV" },
			{ id: 3, name: "New King James Version", abbr: "NKJV", code: "NKJV" },
			{ id: 2, name: "New Living Translation", abbr: "NLT", code: "NLT" },
			{ id: 5, name: "New International Reader's Version", abbr: "NIRV", code: "NIRV" },
			{ id: 28, name: "New Revised Standard Version", abbr: "NRSV", code: "NRSV" },
			{ id: 16, name: "Novum Testamentum Graece", abbr: "NTG", code: "TISCH" },
			{ id: 17, name: "Novum Testamentum Graece (Tischendorf)", abbr: "NTG-T", code: "TISCHENDORF" },
			{ id: 18, name: "Podle Puvodního Vydání Kralického", abbr: "PPVK", code: "CS-KR1579" },
			{ id: 27, name: "Revised Standard Version", abbr: "RSV", code: "RSV" },
			{ id: 10, name: "Stephen's Textus Receptus (1550)", abbr: "STEPHEN", code: "STEPHENS" },
			{ id: 6, name: "Today's New International Version", abbr: "TNIV", code: "TNIV" },
			{ id: 19, name: "Scrivener's Textus Receptus (1881)", abbr: "TR1881", code: "TR1881" },
			{ id: 20, name: "Scrivener's Textus Receptus (1894)", abbr: "TR1894", code: "TR1894MR" },
			{ id: 21, name: "Stephens's Textus Receptus (1550)", abbr: "TR1550", code: "TR1550MR" },
			{ id: 22, name: "Szent Biblia", abbr: "SZENT", code: "KAR" },
			{ id: 29, name: "The Message", abbr: "MSG", code: "MESSAGE" },
			{ id: 12, name: "Young's Literal Translation", abbr: "YLT", code: "YLT" }
		]
	},
	{
		id: 1,
		name: "Crosswalk Interlinear Bible",
		excludeURL: "http://bible.crosswalk.com",
		refSearchURL: "http://bible.crosswalk.com/InterlinearBible/bible.cgi?new=1&word=%R&section=0&version=%V",
		refFixType: 0,
		versions: [
			{ id: 0, name: "KJV Strong's Version", abbr: "KJV", code: "kjv" },
			{ id: 1, name: "NAS Strong's Version", abbr: "NAS", code: "nas" }
		]
	},
	{
		id: 2,
		name: "Crosswalk Online Study Bible",
		excludeURL: "http://bible.crosswalk.com",
		refSearchURL: "http://bible.crosswalk.com/OnlineStudyBible/bible.cgi?new=1&word=%R&section=0&version=%V",
		refFixType: 0,
		versions: [
			{ id: 1, name: "American Standard Version", abbr: "ASV", code: "asv" },
			{ id: 12, name: "Douay-Rheims Bible", abbr: "DRB", code: "rhe" },
			{ id: 2, name: "English Standard Version", abbr: "ESV", code: "esv" },
			{ id: 18, name: "GOD'S WORD Translation", abbr: "GWD", code: "gwd" },
			{ id: 11, name: "Good News Translation", abbr: "GNT", code: "gnt" },
			{ id: 19, name: "Hebrew Names Version", abbr: "HNV", code: "hnv" },
			{ id: 5, name: "Holman Christian Standard Bible", abbr: "HCSB", code: "csb" },
			{ id: 4, name: "King James Version", abbr: "KJV", code: "kjv" },
			{ id: 13, name: "KJV with Strong's Numbers", abbr: "KJV", code: "str" },
			{ id: 14, name: "NAS with Strong's Numbers", abbr: "NAS", code: "nsn" },
			{ id: 0, name: "New American Standard", abbr: "NAS", code: "nas" },
			{ id: 17, name: "New Century Version", abbr: "NCV", code: "ncv" },
			{ id: 7, name: "New International Version", abbr: "NIV", code: "niv" },
			{ id: 3, name: "New King James Version", abbr: "NKJ", code: "nkj" },
			{ id: 8, name: "New Living Translation", abbr: "NLT", code: "nlt" },
			{ id: 9, name: "New Revised Standard", abbr: "NRS", code: "nrs" },
			{ id: 10, name: "Revised Standard Version", abbr: "RSV", code: "rsv" },
			{ id: 21, name: "The Bible in Basic English", abbr: "BBE", code: "bbe" },
			{ id: 16, name: "The Complete Jewish Bible", abbr: "CJB", code: "cjb" },
			{ id: 24, name: "The Darby Translation", abbr: "DBY", code: "dby" },
			{ id: 26, name: "The Latin Vulgate", abbr: "VUL", code: "vul" },
			{ id: 15, name: "The Message", abbr: "MSG", code: "msg" },
			{ id: 25, name: "The Webster Bible", abbr: "WBT", code: "wbt" },
			{ id: 6, name: "Third Millennium Bible", abbr: "TMB", code: "tmb" },
			{ id: 23, name: "Today's New International Version", abbr: "TNIV", code: "tnv" },
			{ id: 20, name: "World English Bible", abbr: "WEB", code: "web" },
			{ id: 22, name: "Young's Literal Translation", abbr: "YTL", code: "ylt" }
		]
	},
	{
		id: 7,
		name: "CrossWire Sword Library",
		excludeURL: "sword://",
		refSearchURL: "sword://%V/%R",
		refFixType: 0,
		versions: [
			{ id: 1, name: "American King James Version", abbr: "AKJV", code: "AKJV" },
			{ id: 2, name: "American Standard Version (1901)", abbr: "ASV", code: "ASV" },
			{ id: 0, name: "Current Module", abbr: "CUR", code: "" },
			{ id: 3, name: "English Standard Version", abbr: "ESV", code: "ESV" },
			{ id: 4, name: "King James Version", abbr: "KJV", code: "KJV" }
		]
	},
	{
		id: 3,
		name: "ESV Bible Online",
		excludeURL: "http://www.gnpcb.org/esv/",
		refSearchURL: "http://www.gnpcb.org/esv/search/?q=%R",
		refFixType: 0,
		versions: [
			{ id: 0, name: "English Standard Version", abbr: "ESV", code: "" }
		]
	},
	{
		id: 4,
		name: "IBS NIV Bible",
		excludeURL: "http://www.ibs.org/niv/",
		refSearchURL: "http://www.ibs.org/niv/passagesearch.php?passage_request=%R&niv=yes",
		refFixType: 0,
		versions: [
			{ id: 0, name: "New International Version", abbr: "NIV", code: "" }
		]
	},
	{
		id: 5,
		name: "NeXt Bible Learning Environment",
		excludeURL: "http://net.bible.org",
		refSearchURL: "http://net.bible.org/universal.php?s=%R",
		refFixType: 0,
		versions: [
			{ id: 0, name: "NET Bible", abbr: "NET", code: "" }
		]
	}
];

const Refalizer_2BKS = "(?:1|2|I{1,2})\\s*";
const Refalizer_3BKS = "(?:1|2|3|I{1,3})\\s*";

const Refalizer_BOOKS = [
	"Genesis", "Gen?\\.?", "Gn\\.?",
	"Exodus", "Exo?d?\\.?",
	"Leviticus", "Lev?\\.?",
	"Numbers", "Num?\\.?",
	"Deuteronomy", "Deut?\\.?", "Dt\\.?",
	"Joshua", "Josh?\\.?",
	"Judges", "Ju?dg\\.?",
	"Ruth", "Ru\\.?",
	Refalizer_2BKS + "Samuel", Refalizer_2BKS + "Sam?\\.?",
	Refalizer_2BKS + "Kings", Refalizer_2BKS + "Kin?\\.?",
	Refalizer_2BKS + "Chronicles", Refalizer_2BKS + "Chron\\.?", Refalizer_2BKS + "Chr?\\.?",
	"Ezra", "Ezr\\.?",
	"Nehemiah", "Neh?\\.?",
	"Esther", "Esth?\\.?", "Es\\.?",
	"Job",
	"Psalms?", "Psa?s?\\.?",
	"Proverbs", "Pro?v?\\.?",
	"Ecclesiastes", "Ecc?l?\\.?",
	"Song of Solomon", "Song of Songs", "Song",
	"Isaiah", "Isa?\\.?",
	"Jeremiah", "Jer?\\.?",
	"Lamentations", "Lam?\\.?",
	"Ezekiel", "Ezek?\\.?",
	"Daniel", "Da?n\\.?",
	"Hosea", "Hos?\\.?",
	"Joel", "Joe\\.?",
	"Amos", "Am\\.?",
	"Obadiah", "Ob\\.?",
	"Jonah", "Jon\\.?",
	"Micah", "Mic\\.?",
	"Nahum", "Nah?\\.?",
	"Habakkuk", "Hab\\.?",
	"Zephaniah", "Zeph?\\.?",
	"Haggai", "Hagg?\\.?",
	"Zechariah", "Zech?\\.?",
	"Malachi", "Mal\\.?",
	"Matthew", "Matt?\\.?", "Mt\\.?",
	"Mark", "Mk\\.?",
	"Luke", "Lk\\.?",
	"John", "Joh\\.?", "Jn\\.?",
	"Acts", "Ac\\.?",
	"Romans", "Rom?\\.?", "Rm\\.?",
	Refalizer_2BKS + "Corinthians", Refalizer_2BKS + "Cor?\\.?",
	"Galatians", "Gal?\\.?",
	"Ephesians", "Eph?\\.?",
	"Philippians", "Phil?\\.?",
	"Colossians", "Col?\\.?",
	Refalizer_2BKS + "Thessalonians", Refalizer_2BKS + "Thess?\\.?", Refalizer_2BKS + "Th\\.?",
	Refalizer_2BKS + "Timothy", Refalizer_2BKS + "Tim?\\.?",
	"Titus", "Tit\\.?",
	"Philemon", "Philem\\.?",
	"Hebrews", "He?b\\.?",
	"James", "Jam?s?\\.?",
	Refalizer_2BKS + "Peter", Refalizer_2BKS + "Pet?\\.?",
	"Jude", "Jude?\\.?",
	Refalizer_3BKS + "John", Refalizer_3BKS + "Joh\\.?", Refalizer_3BKS + "Jn\\.?",
	"Revelation", "Rev?\\.?"
];

// RE for full reference without period as chapter/verse separator
const Refalizer_FULL_REF =
		"(" + Refalizer_BOOKS.join("|") + ")\\s*" +
		"(?:\\d{1,3}:\\s*)?\\d{1,3}" +
		"(?:[-\u2013](?:\\d{1,3}:\\s*)?\\d{1,3}|ff)?" +
		"(?:,\\s*(?:\\d{1,3}:\\s*)?\\d{1,3}(?:[-\u2013](?:\\d{1,3}:\\s*)?\\d{1,3}|ff)?)*";

// RE for full reference with period as chapter/verse separator
const Refalizer_FULL_REF_WP = // 
		"(" + Refalizer_BOOKS.join("|") + ")\\s*" +
		"(?:\\d{1,3}[:\\.]\\s*)?\\d{1,3}" +
		"(?:[-\u2013](?:\\d{1,3}[:\\.]\\s*)?\\d{1,3}|ff)?" +
		"(?:,\\s*(?:\\d{1,3}[:\\.]\\s*)?\\d{1,3}(?:[-\u2013](?:\\d{1,3}[:\\.]\\s*)?\\d{1,3}|ff)?)*";

// RE for part reference without period as chapter/verse separator
const Refalizer_PART_REF =
		"\\b\\d{1,3}:\\s*\\d{1,3}" +
		"(?:[-\u2013](?:\\d{1,3}:\\s*)?\\d{1,3}|ff)?" +
		"(?:,\\s*(?:\\d{1,3}:\\s*)?\\d{1,3}(?:[-\u2013](?:\\d{1,3}:\\s*)?\\d{1,3}|ff)?)*";

// RE for full reference with period as chapter/verse separator
const Refalizer_PART_REF_WP =
		"\\b\\d{1,3}[:\\.]\\s*\\d{1,3}" +
		"(?:[-\u2013](?:\\d{1,3}[:\\.]\\s*)?\\d{1,3}|ff)?" +
		"(?:,\\s*(?:\\d{1,3}[:\\.]\\s*)?\\d{1,3}(?:[-\u2013](?:\\d{1,3}[:\\.]\\s*)?\\d{1,3}|ff)?)*";

const Refalizer_REGEXP = new RegExp("(?:" + Refalizer_FULL_REF + ")|(?:" + Refalizer_PART_REF + ")");

const Refalizer_REGEXP_WP = new RegExp("(?:" + Refalizer_FULL_REF_WP + ")|(?:" + Refalizer_PART_REF_WP + ")");

// Returns preferences branch for extension
//
function Refalizer_getPrefsBranch() {
	return Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService)
			.getBranch("extensions.refalizer.");
}

// Returns fixed version of Bible reference (used for query)
//
function Refalizer_getFixedReference(ref, refFixType) {
	switch (refFixType) {
	case 0:
		// Change en dashes to regular dashes
		return ref.replace(/\u2013/g, "-");
	case 1:
		// Change en dashes to regular dashes and colons to periods
		return ref.replace(/\u2013/g, "-").replace(/:/g, ".");
	default:
		return ref;
	}
}

// Returns site/version info from IDs
//
function Refalizer_getSiteVersionInfo(sID, vID) {
	for (var sIndex in Refalizer_SITES) {
		if (Refalizer_SITES[sIndex].id == sID) {
			for (var vIndex in Refalizer_SITES[sIndex].versions) {
				if (Refalizer_SITES[sIndex].versions[vIndex].id == vID) {
					return {
						site: Refalizer_SITES[sIndex],
						version: Refalizer_SITES[sIndex].versions[vIndex]
					};
				}
			}
			return {
				site: Refalizer_SITES[sIndex],
				version: Refalizer_SITES[sIndex].versions[0]
			};
		}
	}

	return {
		site: Refalizer_SITES[0],
		version: Refalizer_SITES[0].versions[0]
	};
}
